<% if @restore_command -%>
restore_command = '<%= @restore_command %>'
<% end -%>
<% if @archive_cleanup_command -%>
archive_cleanup_command = '<%= @archive_cleanup_command %>'
<% end -%>
<% if @recovery_end_command -%>
recovery_end_command = '<%= @recovery_end_command %>'
<% end -%>

<% if @recovery_target_name -%>
recovery_target_name = '<%= @recovery_target_name %>'
<% end -%>
<% if @recovery_target_time -%>
recovery_target_time = '<%= @recovery_target_time %>'
<% end -%>
<% if @recovery_target_xid -%>
recovery_target_xid = '<%= @recovery_target_xid %>'
<% end -%>
<% if @recovery_target_inclusive -%>
recovery_target_inclusive = <%= @recovery_target_inclusive %>
<% end -%>
<% if @recovery_target -%>
recovery_target = '<%= @recovery_target %>'
<% end -%>
<% if @recovery_target_timeline -%>
recovery_target_timeline = '<%= @recovery_target_timeline %>'
<% end -%>
<% if @pause_at_recovery_target -%>
pause_at_recovery_target = <%= @pause_at_recovery_target %>
<% end -%>

<% if @standby_mode -%>
standby_mode = <%= @standby_mode %>
<% end -%>
<% if @primary_conninfo -%>
primary_conninfo = '<%= @primary_conninfo  %>'
<% end -%>
<% if @primary_slot_name -%>
primary_slot_name = '<%= @primary_slot_name %>'
<% end -%>
<% if @trigger_file -%>
trigger_file = '<%= @trigger_file %>'
<% end -%>
<% if @recovery_min_apply_delay -%>
recovery_min_apply_delay = <%= @recovery_min_apply_delay %>
<% end -%>
